@@ -1,4 +1,4 @@
-// Copyright 2020 The NATS Authors
+// Copyright 2020-2021 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -107,10 +107,6 @@
 //! let nc = nats::connect("my_server::4222")?;
 //!
 //! // this will create the consumer if it does not exist already.
-//! // consumer must be mut because the `process*` methods perform
-//! // message deduplication using an interval tree, which is
-//! // also publicly accessible via the `Consumer`'s `dedupe_window`
-//! // field.
 //! let mut consumer = Consumer::create_or_open(nc, "my_stream", "existing_or_created_consumer")?;
 //!
 //! // The `Consumer::process` method executes a closure
@@ -172,7 +168,6 @@ use std::{
     convert::TryFrom,
     fmt::Debug,
     io::{self, Error, ErrorKind},
-    iter::DoubleEndedIterator,
     time::Duration,
 };
 
@@ -225,10 +220,7 @@ pub struct PagedIterator<'a, T> {
     done: bool,
 }
 
-impl<'a, T> std::iter::FusedIterator for PagedIterator<'a, T> where
-    T: DeserializeOwned + Debug
-{
-}
+impl<'a, T> std::iter::FusedIterator for PagedIterator<'a, T> where T: DeserializeOwned + Debug {}
 
 impl<'a, T> Iterator for PagedIterator<'a, T>
 where
@@ -248,8 +240,7 @@ where
         })
         .unwrap();
 
-        let res: io::Result<PagedResponse<T>> =
-            self.manager.js_request(&self.subject, &req);
+        let res: io::Result<PagedResponse<T>> = self.manager.js_request(&self.subject, &req);
 
         let mut page = match res {
             Err(e) => {
@@ -291,8 +282,7 @@ impl NatsClient {
                 "the stream name must not be empty",
             ));
         }
-        let subject: String =
-            format!("{}STREAM.CREATE.{}", self.api_prefix(), cfg.name);
+        let subject: String = format!("{}STREAM.CREATE.{}", self.api_prefix(), cfg.name);
         let req = serde_json::ser::to_vec(&cfg)?;
         self.js_request(&subject, &req)
     }
@@ -305,8 +295,7 @@ impl NatsClient {
                 "the stream name must not be empty",
             ));
         }
-        let subject: String =
-            format!("{}STREAM.UPDATE.{}", self.api_prefix(), cfg.name);
+        let subject: String = format!("{}STREAM.UPDATE.{}", self.api_prefix(), cfg.name);
         let req = serde_json::ser::to_vec(&cfg)?;
         self.js_request(&subject, &req)
     }
@@ -335,10 +324,7 @@ impl NatsClient {
     }
 
     /// List `JetStream` consumers for a stream.
-    pub fn list_consumers<S>(
-        &self,
-        stream: S,
-    ) -> io::Result<PagedIterator<'_, ConsumerInfo>>
+    pub fn list_consumers<S>(&self, stream: S) -> io::Result<PagedIterator<'_, ConsumerInfo>>
     where
         S: AsRef<str>,
     {
@@ -349,8 +335,7 @@ impl NatsClient {
                 "the stream name must not be empty",
             ));
         }
-        let subject: String =
-            format!("{}CONSUMER.LIST.{}", self.api_prefix(), stream);
+        let subject: String = format!("{}CONSUMER.LIST.{}", self.api_prefix(), stream);
 
         Ok(PagedIterator {
             subject,
@@ -362,10 +347,7 @@ impl NatsClient {
     }
 
     /// Query `JetStream` stream information.
-    pub fn stream_info<S: AsRef<str>>(
-        &self,
-        stream: S,
-    ) -> io::Result<StreamInfo> {
+    pub fn stream_info<S: AsRef<str>>(&self, stream: S) -> io::Result<StreamInfo> {
         let stream: &str = stream.as_ref();
         if stream.is_empty() {
             return Err(Error::new(
@@ -373,16 +355,12 @@ impl NatsClient {
                 "the stream name must not be empty",
             ));
         }
-        let subject: String =
-            format!("{}STREAM.INFO.{}", self.api_prefix(), stream);
+        let subject: String = format!("{}STREAM.INFO.{}", self.api_prefix(), stream);
         self.js_request(&subject, b"")
     }
 
     /// Purge `JetStream` stream messages.
-    pub fn purge_stream<S: AsRef<str>>(
-        &self,
-        stream: S,
-    ) -> io::Result<PurgeResponse> {
+    pub fn purge_stream<S: AsRef<str>>(&self, stream: S) -> io::Result<PurgeResponse> {
         let stream: &str = stream.as_ref();
         if stream.is_empty() {
             return Err(Error::new(
@@ -413,8 +391,7 @@ impl NatsClient {
         })
         .unwrap();
 
-        let subject =
-            format!("{}STREAM.MSG.DELETE.{}", self.api_prefix(), stream);
+        let subject = format!("{}STREAM.MSG.DELETE.{}", self.api_prefix(), stream);
 
         self.js_request::<DeleteResponse>(&subject, &req)
             .map(|dr| dr.success)
@@ -436,11 +413,7 @@ impl NatsClient {
     }
 
     /// Create a `JetStream` consumer.
-    pub fn create_consumer<S, C>(
-        &self,
-        stream: S,
-        cfg: C,
-    ) -> io::Result<Consumer>
+    pub fn create_consumer<S, C>(&self, stream: S, cfg: C) -> io::Result<Consumer>
     where
         S: AsRef<str>,
         ConsumerConfig: From<C>,
@@ -478,11 +451,7 @@ impl NatsClient {
     }
 
     /// Delete a `JetStream` consumer.
-    pub fn delete_consumer<S, C>(
-        &self,
-        stream: S,
-        consumer: C,
-    ) -> io::Result<bool>
+    pub fn delete_consumer<S, C>(&self, stream: S, consumer: C) -> io::Result<bool>
     where
         S: AsRef<str>,
         C: AsRef<str>,
@@ -514,11 +483,7 @@ impl NatsClient {
     }
 
     /// Query `JetStream` consumer information.
-    pub fn consumer_info<S, C>(
-        &self,
-        stream: S,
-        consumer: C,
-    ) -> io::Result<ConsumerInfo>
+    pub fn consumer_info<S, C>(&self, stream: S, consumer: C) -> io::Result<ConsumerInfo>
     where
         S: AsRef<str>,
         C: AsRef<str>,
@@ -531,12 +496,7 @@ impl NatsClient {
             ));
         }
         let consumer: &str = consumer.as_ref();
-        let subject: String = format!(
-            "{}CONSUMER.INFO.{}.{}",
-            self.api_prefix(),
-            stream,
-            consumer
-        );
+        let subject: String = format!("{}CONSUMER.INFO.{}.{}", self.api_prefix(), stream, consumer);
         self.js_request(&subject, b"")
     }
 
@@ -592,10 +552,6 @@ pub struct Consumer {
     /// to 5ms, which is likely to be far too low for
     /// workloads crossing physical sites.
     pub timeout: Duration,
-
-    /// Contains ranges of processed messages that will be
-    /// filtered out upon future receipt.
-    dedupe_window: IntervalTree,
 }
 
 impl Consumer {
@@ -603,26 +559,15 @@ impl Consumer {
     /// `ConsumerInfo` that may have been returned
     /// from the `nats::Connection::list_consumers`
     /// iterator.
-    pub fn from_consumer_info(
-        ci: ConsumerInfo,
-        nc: NatsClient,
-    ) -> io::Result<Consumer> {
-        Consumer::existing::<String, ConsumerConfig>(
-            nc,
-            ci.stream_name,
-            ci.config,
-        )
+    pub fn from_consumer_info(ci: ConsumerInfo, nc: NatsClient) -> io::Result<Consumer> {
+        Consumer::existing::<String, ConsumerConfig>(nc, ci.stream_name, ci.config)
     }
 
     /// Instantiate a `JetStream` `Consumer`. Performs a check to see if the consumer
     /// already exists, and creates it if not. If you want to use an existing
     /// `Consumer` without this check and creation, use the `Consumer::existing`
     /// method.
-    pub fn create_or_open<S, C>(
-        nc: NatsClient,
-        stream: S,
-        cfg: C,
-    ) -> io::Result<Consumer>
+    pub fn create_or_open<S, C>(nc: NatsClient, stream: S, cfg: C) -> io::Result<Consumer>
     where
         S: AsRef<str>,
         ConsumerConfig: From<C>,
@@ -647,11 +592,7 @@ impl Consumer {
     }
 
     /// Use an existing `JetStream` `Consumer`
-    pub fn existing<S, C>(
-        nc: NatsClient,
-        stream: S,
-        cfg: C,
-    ) -> io::Result<Consumer>
+    pub fn existing<S, C>(nc: NatsClient, stream: S, cfg: C) -> io::Result<Consumer>
     where
         S: AsRef<str>,
         ConsumerConfig: From<C>,
@@ -659,17 +600,11 @@ impl Consumer {
         let stream = stream.as_ref().to_string();
         let cfg = ConsumerConfig::from(cfg);
 
-        let push_subscriber =
-            if let Some(ref deliver_subject) = cfg.deliver_subject {
-                Some(nc.subscribe(deliver_subject)?)
-            } else {
-                None
-            };
-
-        let mut dedupe_window = IntervalTree::default();
-
-        // JetStream starts indexing at 1
-        dedupe_window.mark_processed(0);
+        let push_subscriber = if let Some(ref deliver_subject) = cfg.deliver_subject {
+            Some(nc.subscribe(deliver_subject)?)
+        } else {
+            None
+        };
 
         Ok(Consumer {
             nc,
@@ -677,7 +612,6 @@ impl Consumer {
             cfg,
             push_subscriber,
             timeout: Duration::from_millis(5),
-            dedupe_window,
         })
     }
 
@@ -702,12 +636,6 @@ impl Consumer {
     /// before the entire batch is processed, there will be no error
     /// pushed to the returned `Vec`, it will just be shorter than the
     /// specified batch size.
-    ///
-    /// All messages are deduplicated using the `Consumer`'s built-in
-    /// `dedupe_window` before being fed to the provided closure. If
-    /// a message that has already been processed is received, it will
-    /// be acked and skipped. Errors for acking deduplicated messages
-    /// are not included in the returned `Vec`.
     pub fn process_batch<R, F: FnMut(&Message) -> io::Result<R>>(
         &mut self,
         batch_size: usize,
@@ -732,11 +660,10 @@ impl Consumer {
                 self.cfg.durable_name.as_ref().unwrap()
             );
 
-            let sub =
-                match self.nc.request_multi(&subject, batch_size.to_string()) {
-                    Ok(sub) => sub,
-                    Err(e) => return vec![Err(e)],
-                };
+            let sub = match self.nc.request_multi(&subject, batch_size.to_string()) {
+                Ok(sub) => sub,
+                Err(e) => return vec![Err(e)],
+            };
             _sub_opt = Some(sub);
             _sub_opt.as_ref().unwrap()
         };
@@ -758,15 +685,7 @@ impl Consumer {
                 responses.next_timeout(timeout).ok()
             }
         } {
-            let next_id = next.jetstream_message_info().unwrap().stream_seq;
-
-            if self.dedupe_window.already_processed(next_id) {
-                let _dont_care_about_success = next.ack();
-                continue;
-            }
-
             let ret = f(&next);
-
             let is_err = ret.is_err();
             rets.push(ret);
 
@@ -775,7 +694,6 @@ impl Consumer {
                 // if our ack policy is `All`, after breaking.
                 break;
             } else if self.cfg.ack_policy == AckPolicy::Explicit {
-                self.dedupe_window.mark_processed(next_id);
                 let res = next.ack();
                 if let Err(e) = res {
                     rets.push(Err(e));
@@ -806,66 +724,39 @@ impl Consumer {
     ///
     /// Does not ack the processed message if the internal closure returns an `Err`.
     ///
-    /// All messages are deduplicated using the `Consumer`'s built-in
-    /// `dedupe_window` before being fed to the provided closure. If
-    /// a message that has already been processed is received, it will
-    /// be acked and skipped.
-    ///
-    /// Does not return an `Err` if acking the message is unsuccessful,
-    /// but the message is still marked in the dedupe window. If you
-    /// require stronger processing guarantees, you can manually call
+    /// Does not return an `Err` if acking the message is unsuccessful.
+    /// If you require stronger processing guarantees, you can manually call
     /// the `double_ack` method of the argument message. If you require
     /// both the returned `Ok` from the closure and the `Err` from a
     /// failed ack, use `process_batch` instead.
-    pub fn process<R, F: Fn(&Message) -> io::Result<R>>(
-        &mut self,
-        f: F,
-    ) -> io::Result<R> {
-        loop {
-            let next = if let Some(ps) = &self.push_subscriber {
-                ps.next().unwrap()
-            } else {
-                if self.cfg.durable_name.is_none() {
-                    return Err(Error::new(
-                        ErrorKind::InvalidInput,
-                        "process and process_batch are only usable from \
-                        Pull-based Consumers if there is a durable_name set",
-                    ));
-                }
-
-                let subject = format!(
-                    "{}CONSUMER.MSG.NEXT.{}.{}",
-                    self.api_prefix(),
-                    self.stream,
-                    self.cfg.durable_name.as_ref().unwrap()
-                );
-
-                self.nc.request(&subject, AckKind::Ack)?
-            };
-
-            let next_id = if let Some(jmi) = next.jetstream_message_info() {
-                jmi.stream_seq
-            } else {
+    pub fn process<R, F: Fn(&Message) -> io::Result<R>>(&mut self, f: F) -> io::Result<R> {
+        let next = if let Some(ps) = &self.push_subscriber {
+            ps.next().unwrap()
+        } else {
+            if self.cfg.durable_name.is_none() {
                 return Err(Error::new(
-                    ErrorKind::Other,
-                    "failed to process jetstream message info \
-                    from message reply subject. Is your nats-server up to date?"
+                    ErrorKind::InvalidInput,
+                    "process and process_batch are only usable from \
+                        Pull-based Consumers if there is a durable_name set",
                 ));
-            };
-
-            if self.dedupe_window.already_processed(next_id) {
-                let _dont_care = next.ack();
-                continue;
             }
 
-            let ret = f(&next)?;
-            if self.cfg.ack_policy != AckPolicy::None {
-                let _dont_care = next.ack();
-            }
+            let subject = format!(
+                "{}CONSUMER.MSG.NEXT.{}.{}",
+                self.api_prefix(),
+                self.stream,
+                self.cfg.durable_name.as_ref().unwrap()
+            );
 
-            self.dedupe_window.mark_processed(next_id);
-            return Ok(ret);
+            self.nc.request(&subject, AckKind::Ack)?
+        };
+
+        let ret = f(&next)?;
+        if self.cfg.ack_policy != AckPolicy::None {
+            let _dont_care = next.ack();
         }
+
+        Ok(ret)
     }
 
     /// Process and acknowledge a single message, waiting up to the `Consumer`'s
@@ -873,66 +764,45 @@ impl Consumer {
     ///
     /// Does not ack the processed message if the internal closure returns an `Err`.
     ///
-    /// All messages are deduplicated using the `Consumer`'s built-in
-    /// `dedupe_window` before being fed to the provided closure. If
-    /// a message that has already been processed is received, it will
-    /// be acked and skipped.
-    ///
     /// Does not return an `Err` if acking the message is unsuccessful,
-    /// but the message is still marked in the dedupe window. If you
-    /// require stronger processing guarantees, you can manually call
+    /// If you require stronger processing guarantees, you can manually call
     /// the `double_ack` method of the argument message. If you require
     /// both the returned `Ok` from the closure and the `Err` from a
     /// failed ack, use `process_batch` instead.
-    pub fn process_timeout<R, F: Fn(&Message) -> io::Result<R>>(
-        &mut self,
-        f: F,
-    ) -> io::Result<R> {
-        loop {
-            let next = if let Some(ps) = &self.push_subscriber {
-                ps.next_timeout(self.timeout)?
-            } else {
-                if self.cfg.durable_name.is_none() {
-                    return Err(Error::new(
-                        ErrorKind::InvalidInput,
-                        "process and process_batch are only usable from \
+    pub fn process_timeout<R, F: Fn(&Message) -> io::Result<R>>(&mut self, f: F) -> io::Result<R> {
+        let next = if let Some(ps) = &self.push_subscriber {
+            ps.next_timeout(self.timeout)?
+        } else {
+            if self.cfg.durable_name.is_none() {
+                return Err(Error::new(
+                    ErrorKind::InvalidInput,
+                    "process and process_batch are only usable from \
                         Pull-based Consumers if there is a a durable_name set",
-                    ));
-                }
-
-                let subject = format!(
-                    "{}CONSUMER.MSG.NEXT.{}.{}",
-                    self.api_prefix(),
-                    self.stream,
-                    self.cfg.durable_name.as_ref().unwrap()
-                );
-
-                self.nc.request_timeout(&subject, b"", self.timeout)?
-            };
-
-            let next_id = next.jetstream_message_info().unwrap().stream_seq;
-
-            if self.dedupe_window.already_processed(next_id) {
-                self.dedupe_window.mark_processed(next_id);
-                let _dont_care = next.ack();
-                continue;
+                ));
             }
 
-            let ret = f(&next)?;
-            if self.cfg.ack_policy != AckPolicy::None {
-                let _dont_care = next.ack();
-            }
-            return Ok(ret);
+            let subject = format!(
+                "{}CONSUMER.MSG.NEXT.{}.{}",
+                self.api_prefix(),
+                self.stream,
+                self.cfg.durable_name.as_ref().unwrap()
+            );
+
+            self.nc.request_timeout(&subject, b"", self.timeout)?
+        };
+
+        let ret = f(&next)?;
+        if self.cfg.ack_policy != AckPolicy::None {
+            let _dont_care = next.ack();
         }
+
+        Ok(ret)
     }
 
     /// For pull-based consumers (a consumer where `ConsumerConfig.deliver_subject` is `None`)
     /// this can be used to request a single message, and wait forever for a response.
     /// If you require specifying the batch size or using a timeout while consuming the
     /// responses, use the `pull_opt` method below.
-    ///
-    /// This is a lower-level method and does not filter messages through the `Consumer`'s
-    /// built-in `dedupe_window` as the various `process*` methods do.
     pub fn pull(&mut self) -> io::Result<Message> {
         let ret_opt = self
             .pull_opt(NextRequest {
@@ -955,13 +825,7 @@ impl Consumer {
     /// this can be used to request a configurable number of messages, as well as specify
     /// how the server will keep track of this batch request over time. See the docs for
     /// `NextRequest` for more information about the options.
-    ///
-    /// This is a lower-level method and does not filter messages through the `Consumer`'s
-    /// built-in `dedupe_window` as the various `process*` methods do.
-    pub fn pull_opt(
-        &mut self,
-        next_request: NextRequest,
-    ) -> io::Result<crate::Subscription> {
+    pub fn pull_opt(&mut self, next_request: NextRequest) -> io::Result<crate::Subscription> {
         if self.cfg.durable_name.is_none() {
             return Err(Error::new(
                 ErrorKind::InvalidInput,
@@ -993,87 +857,3 @@ impl Consumer {
         &self.nc.0.client.options.jetstream_prefix
     }
 }
-
-/// Records ranges of acknowledged IDs for
-/// low-memory deduplication.
-#[derive(Default)]
-struct IntervalTree {
-    // stores interval start-end
-    inner: std::collections::BTreeMap<u64, u64>,
-}
-
-impl IntervalTree {
-    /// Mark this ID as being processed. Returns `true`
-    /// if this ID was not already marked as processed.
-    pub fn mark_processed(&mut self, id: u64) -> bool {
-        if self.inner.is_empty() {
-            self.inner.insert(id, id);
-            return true;
-        }
-
-        let (prev_start, prev_end) = self
-            .inner
-            .range(..=&id)
-            .next_back()
-            .map(|(s, e)| (*s, *e))
-            .unwrap();
-
-        if (prev_start..=prev_end).contains(&id) {
-            // range already includes id
-            return false;
-        }
-
-        // we may have to merge one or two ranges.
-        //
-        // say we're inserting 4:
-        //
-        // case 1, fully disjoint:
-        // [0] [4] [6]
-        //
-        // case 2, left merge
-        // [0, 1, 2, 3, 4] [6]
-        //
-        // case 3, right merge
-        // [0] [4, 5, 6]
-        //
-        // case 4, double merge
-        // [3, 4, 5]
-
-        let left_merge = prev_end + 1 == id;
-        let right_merge = self.inner.contains_key(&(id + 1));
-
-        match (left_merge, right_merge) {
-            (true, true) => {
-                let right_end = self.inner.remove(&(id + 1)).unwrap();
-                assert_eq!(
-                    self.inner.insert(prev_start, right_end),
-                    Some(id - 1)
-                );
-            }
-            (true, false) => {
-                assert_eq!(self.inner.insert(prev_start, id), Some(id - 1));
-            }
-            (false, true) => {
-                let right_end = self.inner.remove(&(id + 1)).unwrap();
-                assert_eq!(self.inner.insert(id, right_end), None);
-            }
-            (false, false) => {
-                // created disjoint range
-                self.inner.insert(id, id);
-            }
-        }
-
-        true
-    }
-
-    /// Returns `true` if this ID has already been processed.
-    pub fn already_processed(&self, id: u64) -> bool {
-        if let Some((prev_start, prev_end)) =
-            self.inner.range(..=&id).next_back()
-        {
-            (prev_start..=prev_end).contains(&&id)
-        } else {
-            false
-        }
-    }
-}
